package Bencher::Scenario::Accessors::GeneratorStartup;

our $DATE = '2016-06-26'; # DATE
our $VERSION = '0.08'; # VERSION

use Bencher::ScenarioUtil::Accessors;

my $classes = \%Bencher::ScenarioUtil::Accessors::classes;

our $scenario = {
    summary => 'Benchmark startup of various accessor generators',
    module_startup => 1,
    modules => {
    },
    participants => [
        map {
            my $spec = $classes->{$_};
            +{ (module=>$spec->{generator}) x !!$spec->{generator} };
        } keys %$classes,
    ],
};

1;
# ABSTRACT: Benchmark startup of various accessor generators

__END__

=pod

=encoding UTF-8

=head1 NAME

Bencher::Scenario::Accessors::GeneratorStartup - Benchmark startup of various accessor generators

=head1 VERSION

This document describes version 0.08 of Bencher::Scenario::Accessors::GeneratorStartup (from Perl distribution Bencher-Scenarios-Accessors), released on 2016-06-26.

=head1 SYNOPSIS

To run benchmark with default option:

 % bencher -m Accessors::GeneratorStartup

For more options (dump scenario, list/include/exclude/add participants, list/include/exclude/add datasets, etc), see L<bencher> or run C<bencher --help>.

=head1 BENCHMARKED MODULES

Version numbers shown below are the versions used when running the sample benchmark.

L<Class::Accessor> 0.34

L<Class::Accessor::Array> 0.02

L<Class::InsideOut> 1.13

L<Class::XSAccessor> 1.19

L<Class::XSAccessor::Array> 1.19

L<Mo> 0.39

L<Mojo::Base>

L<Moo> 2.001001

L<Moose> 2.1605

L<Mouse> v2.4.5

L<Object::Simple> 3.16

L<Simple::Accessor> 1.02

=head1 BENCHMARK PARTICIPANTS

=over

=item * Mo (perl_code)

L<Mo>



=item *  (perl_code)



=item * Class::XSAccessor (perl_code)

L<Class::XSAccessor>



=item * Class::InsideOut (perl_code)

L<Class::InsideOut>



=item * Simple::Accessor (perl_code)

L<Simple::Accessor>



=item * Mouse (perl_code)

L<Mouse>



=item * Class::XSAccessor::Array (perl_code)

L<Class::XSAccessor::Array>



=item * Moose (perl_code)

L<Moose>



=item *  (perl_code)



=item * Class::Accessor (perl_code)

L<Class::Accessor>



=item * Mojo::Base (perl_code)

L<Mojo::Base>



=item * Class::Accessor::Array (perl_code)

L<Class::Accessor::Array>



=item * Moo (perl_code)

L<Moo>



=item *  (perl_code)



=item * Object::Simple (perl_code)

L<Object::Simple>



=back

=head1 SAMPLE BENCHMARK RESULTS

Run on: perl: I<< v5.22.2 >>, CPU: I<< Intel(R) Core(TM) i7-4770 CPU @ 3.40GHz (4 cores) >>, OS: I<< GNU/Linux Debian version 8.0 >>, OS kernel: I<< Linux version 3.16.0-4-amd64 >>.

Benchmark with default options (C<< bencher -m Accessors::GeneratorStartup >>):

 +--------------------------+-----------+------------------------+------------+-----------+---------+
 | participant              | time (ms) | mod_overhead_time (ms) | vs_slowest |  errors   | samples |
 +--------------------------+-----------+------------------------+------------+-----------+---------+
 | Moose                    |     120   |                  117.4 |        1   |   0.00029 |      20 |
 | Mouse                    |      18   |                   15.4 |        6.8 | 5.7e-05   |      20 |
 | Moo                      |      14   |                   11.4 |        8.9 | 6.4e-05   |      20 |
 | Class::InsideOut         |      11   |                    8.4 |       11   | 7.6e-05   |      20 |
 | Mojo::Base               |      10   |                    7.4 |       11   | 5.2e-05   |      21 |
 | Class::XSAccessor::Array |       8.2 |                    5.6 |       15   | 5.9e-05   |      20 |
 | Object::Simple           |       8.2 |                    5.6 |       15   | 4.2e-05   |      20 |
 | Class::Accessor          |       7.7 |                    5.1 |       16   | 6.3e-05   |      20 |
 | Class::XSAccessor        |       7.7 |                    5.1 |       16   | 4.2e-05   |      20 |
 | Simple::Accessor         |       5   |                    2.4 |       30   | 5.2e-05   |      20 |
 | Class::Accessor::Array   |       4.3 |                    1.7 |       28   | 3.6e-05   |      20 |
 | Mo                       |       4.2 |                    1.6 |       28   | 4.2e-05   |      23 |
 | perl -e1 (baseline)      |       2.6 |                    0   |       47   | 7.2e-06   |      20 |
 +--------------------------+-----------+------------------------+------------+-----------+---------+

=head1 DESCRIPTION

Packaging a benchmark script as a Bencher scenario makes it convenient to include/exclude/add participants/datasets (either via CLI or Perl code), send the result to a central repository, among others . See L<Bencher> and L<bencher> (CLI) for more details.

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/Bencher-Scenarios-Accessors>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-Bencher-Scenarios-Accessors>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Bencher-Scenarios-Accessors>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 SEE ALSO

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2016 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
